"""Main interactive menu for pyattck."""
from ..base import Base
from .layout import CustomLayout
from .menu import Menu


class Interactive(Base):
    """Generates the interactive menu, options, and drives the display of the menu system."""

    _framework = None
    _type = None
    _object = None

    def __init__(self, attck_instance) -> None:
        """A pyattck Attck instance class."""
        self._attck_instance = attck_instance

    def generate(self):
        """Generates the interactive console for pyattck."""
        main_menu = Menu()
        main_menu.prompt = "Select the appropriate MITRE ATT&CK Framework:\n"
        for framework in self.FRAMEWORKS:
            obj_menu = Menu()
            obj_menu.prompt = "Select an entity below:\n"
            for obj in dir(getattr(self._attck_instance, framework)):
                if not obj.startswith("_") and obj in self.ATTCK_TYPES:
                    item_menu = Menu()
                    for item in getattr(getattr(self._attck_instance, framework), obj):
                        item_menu.add_option(getattr(item, "name"), CustomLayout(item))
                    obj_menu.add_option(obj, item_menu)
            main_menu.add_option(framework, obj_menu, True)
        main_menu.run()



class GeneratedDatasetException(Exception):
    """Raised when unable to retrieve generated datasets and related properties."""

    pass


class ConfigurationException(Exception):
    """Raised when unable to load or read the pyattck configuration file."""

    pass


class UnknownFileError(ValueError):
    """Raised when the provided file extension is unkown or is not json, yml or yaml."""

    def __init__(self, provided_value=None, known_values=None):
        if provided_value and known_values:
            if isinstance(known_values, list):
                super().__init__(
                    f"The provided value {provided_value} is unknown. "
                    f"Please provide a file path with one of these '{[x for x in known_values]}' extensions."
                )
        else:
            pass



"""Menu for interactive console within pyattck."""
import base64
import os
from typing import List

from ..base import Base


class Menu(Base):
    """Menu to control flow of the interactive menu."""

    def __init__(self):
        """Creates a new instance of a Menu class object."""
        self.options = {}
        self._option_number = 0
        self._selected_option = None
        self.prompt = "Please choose an option:"
        self.error_text = "Error! Please enter a valid option."
        self._logo_displayed = False

    @property
    def prompt(self) -> str:
        """The prompt question to use when using this menu."""
        return self._prompt

    @prompt.setter
    def prompt(self, value: str) -> None:
        """Sets the prompt question to use when using this menu."""
        self._prompt = value

    @property
    def error_text(self) -> str:
        """The error text for this menu."""
        return self._error_text

    @error_text.setter
    def error_text(self, value: str) -> None:
        """Sets the error text for the current menu."""
        self._error_text = value

    @property
    def selected_option(self) -> List[str]:
        """Returns the currently selected option."""
        if self._selected_option:
            return self.options[self._selected_option]
        return None

    @selected_option.setter
    def selected_option(self, value: int) -> None:
        """Sets the currently selected option from the options list."""
        self._selected_option = value

    def cls(self) -> None:
        """Clears the current screen."""
        os.system("cls" if os.name == "nt" else "clear")

    def display_error(self) -> None:
        """Displays the defined error message with padding as needed."""
        print(f"\n{self.error_text}\n")

    def display_menu(self, clear_screen: bool = False) -> None:
        """Displays the current menu options."""
        if clear_screen:
            self.cls()
        print(self.prompt)
        for i in range(1, len(self.options) + 1):
            print(f"{i} - {self.options[i][0]}")

    def add_option(self, name: str, option: object or None, triggers_exit: bool = False) -> None:
        """Adds an option to the menu option list."""
        self._option_number += 1
        self.options[self._option_number] = [name, option, triggers_exit]

    def run(self) -> None:
        """Main method to display the current menu, as well as all provided sub-menus."""
        user_input = ""
        if not self._logo_displayed:
            self.cls()
            print(base64.b64decode(self.LOGO).decode("ascii"))
            self._logo_displayed = True
        self.display_menu()
        while True:
            user_input = input("\nYour selection: ")
            try:
                user_input = int(user_input)
                if user_input <= 0 or user_input > len(self.options):
                    self.display_error()
                else:
                    if callable(self.options[user_input][1]):
                        self.selected_option = user_input
                        self.options[user_input][1]()
                        if self.options[user_input][2]:
                            return
                        else:
                            self.display_menu()
                    elif self.options[user_input][1]:
                        self.selected_option = user_input
                        self.options[user_input][1].run()
                        self.display_menu(clear_screen=True)
                    else:
                        return
            except ValueError:
                self.display_error()



import json
import os
import warnings

import yaml
from attrs import asdict, define, field
from pydantic import DirectoryPath, FilePath, HttpUrl
from requests.api import request

from .utils.exceptions import UnknownFileError
from .utils.utils import get_absolute_path, is_path, is_url


@define
class Configuration:
    data_path: DirectoryPath = field(default="~/pyattck/data", converter=get_absolute_path)
    enterprise_attck_json: HttpUrl or FilePath = field(
        default="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_enterprise_attck_v1.json",
        converter=get_absolute_path,
    )
    pre_attck_json: HttpUrl or FilePath = field(
        default="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_pre_attck_v1.json",
        converter=get_absolute_path,
    )
    mobile_attck_json: HttpUrl or FilePath = field(
        default="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_mobile_attck_v1.json",
        converter=get_absolute_path,
    )
    ics_attck_json: HttpUrl or FilePath = field(
        default="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_ics_attck_v1.json",
        converter=get_absolute_path,
    )
    nist_controls_json: HttpUrl or FilePath = field(
        default="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_nist_controls_v1.json",
        converter=get_absolute_path,
    )
    generated_nist_json: HttpUrl or FilePath = field(
        default="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/attck_to_nist_controls.json",
        converter=get_absolute_path,
    )

    @enterprise_attck_json.validator
    @pre_attck_json.validator
    @mobile_attck_json.validator
    @ics_attck_json.validator
    @nist_controls_json.validator
    @generated_nist_json.validator
    def _validate_json_value(self, attribute, value):
        path_valid = False
        url_valid = False
        path_valid = is_path(value)
        url_valid = is_url(value)
        if not path_valid and not url_valid:
            raise Exception("The provided value is neither a URL or file path")


@define(frozen=True)
class Options:
    nested_techniques: bool = field(default=False)
    use_config: bool = field(default=False)
    save_config: bool = field(default=False)
    config_file_path: FilePath = field(default="~/pyattck/config.yml", converter=get_absolute_path)
    config: Configuration = field(factory=Configuration)
    kwargs: dict = field(factory=dict)

    def _download_url_data(self, url):
        response = request("GET", url, **self.kwargs)
        if response.status_code == 200:
            return response.json()
        return {}

    def _read_from_disk(self, path):
        if os.path.exists(path) and os.path.isfile(path):
            try:
                with open(path) as f:
                    if path.endswith(".json"):
                        return json.load(f)
                    elif path.endswith(".yml") or path.endswith(".yaml"):
                        return Configuration(**yaml.load(f, Loader=yaml.SafeLoader))
                    else:
                        raise UnknownFileError(provided_value=path, known_values=[".json", ".yml", ".yaml"])
            except Exception as e:
                warnings.warn(
                    message=f"The provided config file {path} is not in the correct format. "
                    "Using default values instead."
                )
                pass
        elif os.path.isdir(path):
            raise Exception(f"The provided path is a directory and must be a file: {path}")

    def _save_to_disk(self, path, data):
        try:
            if not os.path.exists(os.path.dirname(path)):
                try:
                    os.makedirs(os.path.dirname(path))
                except Exception as e:
                    raise Exception(
                        "pyattck attempted to create the provided directories but was unable to: {}".format(path)
                    )
            with open(path, "w+") as f:
                if path.endswith(".json"):
                    json.dump(data, f)
                elif path.endswith(".yml") or path.endswith(".yaml"):
                    yaml.dump(data, f)
                else:
                    raise UnknownFileError(provided_value=path, known_values=[".json", ".yml", ".yaml"])
        except IsADirectoryError as ie:
            raise Exception(f"The provided path is a directory: {path}")

    def _save_json_data(self, force: bool = False) -> None:
        if not os.path.exists(self.config.data_path):
            try:
                os.makedirs(self.config.data_path)
            except Exception as e:
                raise Exception("Unable to save data to the provided location: {}".format(self.config.data_path))
        for json_data in [
            "enterprise_attck_json",
            "pre_attck_json",
            "mobile_attck_json",
            "ics_attck_json",
            "nist_controls_json",
            "generated_nist_json",
        ]:
            if is_url(getattr(self.config, json_data)):
                try:
                    path = os.path.join(self.config.data_path, f"{json_data}.json")
                    if not os.path.exists(path) or force:
                        data = self._download_url_data(getattr(self.config, json_data))
                        self._save_to_disk(path, data)
                except Exception as e:
                    raise Warning(f"Unable to download data from {json_data}")
        return True

    def get_data(self, value: str) -> dict:
        """Retrieves saved data based on key value in config.

        Args:
            value (str): A key value in our configuration file.

        Returns:
            dict: The dictionary object which was retrieved.
        """
        data = getattr(self.config, value)
        if is_url(data):
            return self._download_url_data(data)
        else:
            return self._read_from_disk(getattr(self.config, value))

    def _save_config(self, config_file_path: str, config_dict: dict) -> None:
        """Saves the config to the provided path."""
        self._save_to_disk(config_file_path, config_dict)
        self._save_json_data()

    def __attrs_post_init__(self):
        """Contains options and configuration for pyattck."""
        if self.save_config:
            self._save_config(config_file_path=self.config_file_path, config_dict=asdict(self.config))
        if self.use_config:
            self._save_config(config_file_path=self.config_file_path, config_dict=asdict(self.config))
            object.__setattr__(self, "config", self._read_from_disk(self.config_file_path))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyattck', 'pyattck.utils']

package_data = \
{'': ['*'], 'pyattck': ['data/*']}

install_requires = \
['attrs>=21.4.0,<22.0.0',
 'fire>=0.4.0,<0.5.0',
 'pyattck-data>=2.5.0,<3.0.0',
 'requests>=2.27.1,<3.0.0',
 'rich>=12.5.1,<13.0.0']

entry_points = \
{'console_scripts': ['pyattck = pyattck.cli:main']}

setup_kwargs = {
    'name': 'pyattck',
    'version': '7.1.1',
    'description': 'A Python package to interact with the Mitre ATT&CK Frameworks',
    'long_description': '![pyattck](https://github.com/swimlane/pyattck/workflows/Testing%20pyattck/badge.svg)\n![](./images/ubuntu_support.svg)\n![](./images/macos_support.svg)\n![](./images/windows_support.svg)\n![](./images/code_coverage.svg)\n\n# Welcome to pyattck\'s Documentation\n\n```\n    .______   ____    ____  ___   .___________.___________.  ______  __  ___\n    |   _  \\  \\   \\  /   / /   \\  |           |           | /      ||  |/  /\n    |  |_)  |  \\   \\/   / /  ^  \\ `---|  |----`---|  |----`|  ,----\'|  \'  /\n    |   ___/    \\_    _/ /  /_\\  \\    |  |        |  |     |  |     |    <\n    |  |          |  |  /  _____  \\   |  |        |  |     |  `----.|  .  \\\n    | _|          |__| /__/     \\__\\  |__|        |__|      \\______||__|\\__\\\n\n```\n\tA Python package to interact with MITRE ATT&CK Frameworks\n\n> Current Version is 7.1.1\n\n**pyattck** is a light-weight framework for MITRE ATT&CK Frameworks. This package extracts details from the MITRE Enterprise, PRE-ATT&CK, Mobile, and ICS Frameworks.\n\n## Why?\n\n`pyattck` assist organizations and individuals with accessing MITRE ATT&CK Framework(s) in a programmatic way. Meaning, you can access all defined actors, malwares, mitigations, tactics, techniques, and tools defined by the Enterprise, Mobile, Pre-Attck, and ICS frameworks via a command-line utility or embedding into your own code base.\n\nThere are many reasons why you would want to access this data in an automated (scripted/coded) way but a few examples are:\n\n* Generate reports with additional details about a technique (or any object defined in the framework) \n* A build pipeline of detection rules with additional MITRE ATT&CK details for categorization\n* Quickly searching for specific details about a technique without navigating a web page\n\nThere are other benefits that `pyattck` provide as well which includes the ability to provide additional contextual data. You can find more information about this data [here](https://github.com/swimlane/pyattck-data) but the basics are that `pyattck` utilizes multiple open-source repositories to gather additional contextual data like commands used to execute a technique, country and other details about a malicious actor, other variants of malware similar to a defined tool/malware, etc. \n\nThis additional context is what makes `pyattck` truly powerful and enables people to build more robust testing and validation of their detection rules, validates testing assumptions, etc. Truly there are countless ways that `pyattck` could be utilized to help blue, red, and purple teams defend organizations (and themselves).\n\n## Features\n\nThe **pyattck** package retrieves all Tactics, Techniques, Actors, Malware, Tools, and Mitigations from the MITRE ATT&CK Frameworks as well as any defined relationships within the MITRE ATT&CK dataset (including sub-techniques).\n\nIn addition, Techniques, Actors, and Tools (if applicable) now have collected data from third-party resources that are accessible via different properties. For more detailed information about these features, see [External Datasets](https://github.com/swimlane/pyattck-data).\n\nThe **pyattck** package allows you to:\n\n  * Specify a URL or local file path for the MITRE ATT&CK Enterprise Framework json, generated dataset, and/or a config.yml file.\n  * Access data from the MITRE PRE-ATT&CK Framework\n  * Access data from the MITRE Mobile ATT&CK Framework\n  * Access data from the MITRE ICS ATT&CK Framework\n  * Access sub-techniques as nested objects or you can turn it off and access as normal technique\n  * Access compliance controls (currently NIST 800-53 v5) related to a MITRE ATT&CK Technique\n  * pyattck now utilizes structured data models. More information can be found at [pyattck-data](https://github.com/swimlane/pyattck-data)\n  * Run an interactive console menu system to access pyattck data\n\n# Table of Contents\n\n1. [Installation](#installation)\n2. [Usage Example](#usage-example)\n3. [Configuration](#configuration)\n4. [Notes](#note)\n\n## Installation\n\nYou can install **pyattck** on OS X, Linux, or Windows. You can also install it directly from the source. To install, see the commands under the relevant operating system heading, below.\n\n### macOS, Linux and Windows:\n\n```bash\npip install pyattck\n```\n\n### Installing from source\n\n```bash\ngit clone https://github.com/swimlane/pyattck.git\ncd pyattck\npython setup.py install\n```\n\n## Usage example\n\nTo use **pyattck** you must instantiate an **Attck** object. Although you can interact directly with each class, the intended use is through a **Attck** object:\n\n```python\nfrom pyattck import Attck\n\nattack = Attck()\n```\n\nBy default, `sub-techniques` are accessible under each technique object. You can turn this behavior off by passing `nested_techniques=False` when creating your `Attck` object.\n\nAs an example, the default behavior looks like the following example:\n\n```python\nfrom pyattck import Attck\n\nattack = Attck()\n\nfor technique in attack.enterprise.techniques:\n    print(technique.id)\n    print(technique.name)\n    for subtechnique in technique.techniques:\n        print(subtechnique.id)\n        print(subtechnique.name)\n```\n\nYou can access the following `main` properties on your **Attck** object:\n\n* enterprise\n* preattack\n* mobile\n* ics\n\nOnce you specify the MITRE ATT&CK Framework, you can access additional properties.\n\nHere are the accessible objects under the [Enterprise](docs/enterprise.md) property:\n\n* [actors](docs/actor.md)\n* [controls](docs/control.md)\n* [malwares](docs/malware.md)\n* [mitigations](docs/mitigation.md)\n* [tactics](docs/tactic.md)\n* [techniques](docs/technique.md)\n* [tools](docs/tools.md)\n\nFor more information on object types under the `enterprise` property, see [Enterprise](docs/enterprise.md).\n\nHere are the accessible objects under the [PreAttck](docs/preattck.md) property:\n\n* [actors](docs/actor.md)\n* [tactics](docs/tactic.md)\n* [techniques](docs/technique.md)\n\nFor more information on object types under the `preattck` property, see [PreAttck](docs/preattck.md).\n\nHere are the accessible objects under the [Mobile](docs/mobile.md) property:\n\n* [actors](docs/actor.md)\n* [malwares](docs/malware.md)\n* [mitigations](docs/mitigation.md)\n* [tactics](docs/tactic.md)\n* [techniques](docs/technique.md)\n* [tools](docs/tools.md)\n\nFor more information on object types under the `mobile` property, see [Mobile](docs/mobile.md).\n\nHere are the accessible objects under the [ICS](docs/ics.md) property:\n\n* [controls](docs/control.md)\n* [malwares](docs/malware.md)\n* [mitigations](docs/mitigation.md)\n* [tactics](docs/tactic.md)\n* [techniques](docs/technique.md)\n\nFor more information on object types under the `ics` property, see [ICS](docs/ics.md).\n\n## Interactive Menu Usage\n\nTo utilize the new interactive menu system within pyattck, you must set `interactive` to `True`. By doing so, it will launch the interactive console menu system.\n\nUsing a script your can launch this by running:\n\n```python\nfrom pyattck import Attck\n\nAttck(interactive=True)\n```\n\nOr you can also run interactive mode on the command line:\n\n```bash\npyattck --interactive\n```\n\nCheckout a gif example below:\n\n![](images/pyattck_interactive_menu.gif)\n\n## Configuration\n\n`pyattck` allows you to configure if you store external data and where it is stored. \n\n```python\nfrom pyattck import Attck\n\nattck = Attck(\n    nested_techniques=True,\n    use_config=False,\n    save_config=False,\n    config_file_path=\'~/pyattck/config.yml\',\n    data_path=\'~/pyattck/data\',\n    enterprise_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_enterprise_attck_v1.json",\n    pre_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_pre_attck_v1.json",\n    mobile_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_mobile_attck_v1.json,\n    ics_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_ics_attck_v1.json",\n    nist_controls_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_nist_controls_v1.json",\n    generated_nist_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/attck_to_nist_controls.json",\n    **kwargs\n)\n```\n\nBy default, `pyattck` will (now) pull the latest external data from their respective locations using HTTP GET requests. `pyattck` currently pulls from the following locations:\n\n* enterprise_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_enterprise_attck_v1.json"\n* pre_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_pre_attck_v1.json"\n* mobile_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_mobile_attck_v1.json"\n* ics_attck_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_ics_attck_v1.json"\n* nist_controls_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/merged_nist_controls_v1.json"\n* generated_nist_json="https://swimlane-pyattck.s3.us-west-2.amazonaws.com/attck_to_nist_controls.json"\n\nYou have several options when instantiating the `Attck` object. As of `4.0.0` you can now specify any of the following options:\n\n* use_config - When you specify this argument as `True` pyattck will attempt to retrieve the configuration specified in the `config_file_path` location. If this file is corrupted or cannot be found, we will default to retrieving data from the specified `*_attck_json` locations.\n* save_config - When you specify this argument as `True` pyattck will save the configuration file to the specified location set by `config_file_path`. Additionally, we will save all downloaded files to the `data_path` location specified. If you have specified a local path location instead of a download URL for any of the `*_attck_json` parameters we will save this location in our configuration and reference this location going forward. \n* config_file_path - The path to store a configuration file. Default is `~/pyattck/config.yml`\n* data_path - The path to store any data files downloaded to the local system. Default is `~/pyattck/data`\n\n### JSON Locations\n\nAdditionally, you can specify the location for each individual `*_attck_json` files by passing in either a URI or a local file path. If you have passed in a local file path, we will simply read from this file. \n\nIf you have used the default values or specified an alternative URI location to retrieve these JSON files from, you can additionally pass in `**kwargs` that will be passed along to the `Requests` python package when performing any HTTP requests.\n\n## Note\n\nWe understand that there are many different open-source projects being released, even on a daily basis, but we wanted to provide a straightforward Python package that allowed the user to identify known relationships between all verticals of the MITRE ATT&CK Framework.\n\nIf you are unfamiliar with the MITRE ATT&CK Framework, there are a few key components to ensure you have a firm grasp around. The first is Tactics & Techniques. When looking at the [MITRE ATT&CK Framework](https://attack.mitre.org/), the Tactics are the columns and represent the different phases of an attack.\n\n   > The MITRE ATT&CK Framework is NOT an all encompassing/defacto security coverage map - it is rather a FRAMEWORK and additional avenues should also be considered when assessing your security posture.\n\nTechniques are the rows of the framework and are categorized underneath specific Tactics (columns). They are data points within the framework that provides guidance when assessing your security gaps. Additionally, (most) Techniques contain mitigation guidance in addition to information about their relationship to tools, malware, and even actors/groups that have used this technique during recorded attacks.\n\nThis means, if your organization is focused on TTPs (Tactics Techniques and Procedures) used by certain actors/groups then MITRE ATT&CK Framework is perfect for you. If you are not at this security maturing within your organization, no worries! The ATT&CK Framework still provides really good guidance in a simple and straightforward layout, but programmatically it is not straightforward--especially if you wanted to measure (or map) your security controls using the framework.\n\n### Developing and Testing\n\nYou can add features or bugs or run the code in a development environment.\n\n1. To get a development and testing environment up and running, use this [Dockerfile](https://github.com/swimlane/pyattck/blob/master/Dockerfile).\n\n2. To use the `Dockerfile` run, cd to this repository directory and run:\n\n  ```\n  docker build --force-rm -t pyattck .\n  ```\n\n3. Next, run the docker container:\n\n  ```\n  docker run pyattck\n  ```\n\n  Running this calls the test python file in [bin/test.py](https://github.com/swimlane/pyattck/blob/master/bin/test.py).\n\n4. Modify the test python file for additional testing and development.\n\n## Running the tests\n\nTests within this project should cover all available properties and methods. As this project grows the tests will become more robust but for now we are testing that they exist and return outputs.\n\n## Contributing\n\nPlease read [CONTRIBUTING.md](https://github.com/swimlane/pyattck/blob/master/CONTRIBUTING.md) for details on our code of conduct, and the process for submitting pull requests to us.\n\n## Versioning\n\nWe use [SemVer](http://semver.org/) for versioning.\n\n## Change Log\n\nFor details on features for a specific version of `pyattck`, see the [CHANGELOG.md](https://github.com/swimlane/pyattck/blob/master/CHANGELOG.md).\n\n## Authors\n\n* Josh Rickard - *Initial work* - [MSAdministrator](https://github.com/msadministrator)\n\nSee also the list of [contributors](https://github.com/swimlane/pyattck/contributors).\n\n## License\n\nThis project is licensed under the [MIT License](https://github.com/swimlane/pyattck/blob/master/LICENSE.md).\n\n## Acknowledgments\n\nFirst of all, I would like to thank everyone who contributes to open-source projects, especially the maintainers and creators of these projects. Without them, this capability would not be possible.\n\nThis data set is generated from many different sources. As we continue to add more sources, we will continue to add them here. Again thank you to all of these projects. In no particular order, `pyattck` utilizes data from the following projects:\n\n* [Mitre ATT&CK APT3 Adversary Emulation Field Manual](https://attack.mitre.org/docs/APT3_Adversary_Emulation_Field_Manual.xlsx)\n* [Atomic Red Team (by Red Canary)](https://github.com/redcanaryco/atomic-red-team)\n* [Atomic Threat Coverage](https://github.com/atc-project/atomic-threat-coverage)\n* [attck_empire (by dstepanic)](https://github.com/dstepanic/attck_empire)\n* [sentinel-attack (by BlueTeamLabs)](https://github.com/BlueTeamLabs/sentinel-attack)\n* [Litmus_test (by Kirtar22)](https://github.com/Kirtar22/Litmus_Test)\n* [nsm-attack (by oxtf)](https://github.com/0xtf/nsm-attack)\n* [osquery-attck (by teoseller)](https://github.com/teoseller/osquery-attck)\n* [Mitre Stockpile](https://github.com/mitre/stockpile)\n* [SysmonHunter (by baronpan)](https://github.com/baronpan/SysmonHunter)\n* [ThreatHunting-Book (by 12306Bro)](https://github.com/12306Bro/Threathunting-book)\n* [threat_hunting_tables (by dwestgard)](https://github.com/dwestgard/threat_hunting_tables)\n* [APT Groups & Operations](https://docs.google.com/spreadsheets/d/1H9_xaxQHpWaa4O_Son4Gx0YOIzlcBWMsdvePFX68EKU/edit#gid=1864660085)\n* [C2Matrix (by @jorgeorchilles, @brysonbort, @adam_mashinchi)](https://www.thec2matrix.com/)\n* [Elemental](https://github.com/Elemental-attack/Elemental)\n* [MalwareArchaeology - ATTACK](https://github.com/MalwareArchaeology/ATTACK)\n* [Attack-Technique-Dataset](https://github.com/NewBee119/Attack-Technique-Dataset)\n\n\n```eval_rst\n.. toctree::\n   :titlesonly:\n\n   configuration\n   pyattck/attck\n   Dataset <https://github.com/swimlane/pyattck-data>\n   Data Models <https://github.com/swimlane/pyattck-data>\n   enterprise/enterprise\n   preattck/preattck\n   mobile/mobileattck\n   ics/icsattck\n```\n',
    'author': 'Swimlane',
    'author_email': 'info@swimlane.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/swimlane/pyattck',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Used to format interactive layout display for individual attack component types."""
from datetime import datetime
from time import sleep

from rich import box
from rich.align import Align
from rich.console import Group
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

from ..base import Base


class Footer(Base):
    """Display footer references."""

    def __init__(self, item):
        """Provided the object type item we generate references and display them on the footer.

        The following attack types are expected:

        "actors","controls","data_components","data_sources","malwares","mitigations","tactics","techniques","tools"

        Args:
            item (Any): A attack type object.
        """
        self.item = item

    def __rich__(self) -> Panel:
        """Generates a Panel object with the correct relationships.

        Returns:
            Panel: A footer relationship Panel object.
        """
        grid = Table.grid(expand=True)
        grid.add_column(justify="center", ratio=1)
        grid.add_column(justify="right")
        panel_list = []
        for item in dir(self.item):
            if not item.startswith("_") and item in self.ATTCK_TYPES:
                if getattr(self.item, item):
                    name_list = set()
                    for i in getattr(self.item, item):
                        name_list.add(getattr(i, "name"))
                    panel_list.append(
                        Panel(
                            ", ".join([x for x in list(name_list)]),
                            title=f"[b]{item}",
                            border_style="red",
                            padding=(1, 2),
                        ),
                    )
        grid.add_row(*panel_list)
        return Panel(grid, style="white on blue")


class Header:
    """Display header with clock."""

    def __init__(self, item):
        """Provided the object type item we generate references and display them on the footer.

        The following attack types are expected:

        "actors","controls","data_components","data_sources","malwares","mitigations","tactics","techniques","tools"

        Args:
            item (Any): A attack type object.
        """
        self.item = item

    def __rich__(self) -> Panel:
        """Returns the header which contains the items name and any aliases.

        Returns:
            Panel: A header Panel object.
        """
        grid = Table.grid(expand=True)
        grid.add_column(justify="center", ratio=1)
        grid.add_column(justify="right")
        if hasattr(self.item, "aliases"):
            text = f"[b]{self.item.name}[/b] - Known Aliases: {', '.join([x for x in self.item.aliases])}"
        else:
            text = f"[b]{self.item.name}[/b]"

        grid.add_row(
            text,
            datetime.now().ctime().replace(":", "[blink]:[/]"),
        )
        return Panel(grid, style="white on blue")


class CustomLayout(Base):
    """Used to generate a custom layout for pyattck object items."""

    def __init__(self, item):
        """Provided the object type item we generate references and display them on the footer.

        The following attack types are expected:

        "actors","controls","data_components","data_sources","malwares","mitigations","tactics","techniques","tools"

        Args:
            item (Any): A attack type object.
        """
        self.item = item
        layout = self.make_layout()
        layout["header"].update(Header(self.item))
        layout["body"].update(self.make_general_information())
        layout["side"].update(Panel(self.make_top_left_box(), title="Details", border_style="red"))
        layout["footer"].update(Footer(self.item))
        self.layout = layout

    def run(self):
        """Main callable for the CustomLayout class.

        This method gets called within the Menu class when a wrapped object is passed to display this
        custom layout within the console.
        """
        with Live(self.layout, refresh_per_second=10, screen=True, redirect_stderr=False) as live:
            try:
                while True:
                    sleep(1)
            except KeyboardInterrupt:
                pass

    def make_layout(self) -> Layout:
        """Defines the console layout."""
        layout = Layout(name="root")

        layout.split(
            Layout(name="header", size=3),
            Layout(name="main", ratio=1),
            Layout(name="footer", size=7),
        )
        layout["main"].split_row(
            Layout(name="side"),
            Layout(name="body", ratio=2, minimum_size=60),
        )
        return layout

    def _get_external_id(self, attck_object) -> str:
        """Retrieves the official MITRE ID from an objects external references.

        Args:
            attck_object (Any): A MITRE ATTCK object type.

        Returns:
            str: The official designated MITRE ATT&CK ID.
        """
        if hasattr(attck_object, "external_references"):
            for item in getattr(attck_object, "external_references"):
                if (
                    hasattr(item, "external_id")
                    and getattr(item, "external_id")
                    and not getattr(item, "external_id").startswith("CAPEC")
                ):
                    return getattr(item, "external_id")

    def make_top_left_box(self):
        """Creates text string used in the Details section."""
        return f"""
[bold cyan1]ID: [/]{self._get_external_id(self.item)}
[bold cyan1]Revoked: [/]{self.item.revoked if hasattr(self.item, 'revoked') else 'False'}
[bold cyan1]Type: [/]{self.item.type}
[bold cyan1]STIX: [/]{self.item.id}
[bold cyan1]Created: [/]{self.item.created}
[bold cyan1]Modified: [/]{self.item.modified}
"""

    def make_general_information(self) -> Panel:
        """Generates the Panel containing the general information section."""
        sponsor_message = Table.grid(padding=1)
        sponsor_message.add_column(style="green", justify="right")
        sponsor_message.add_column(no_wrap=True)

        for ref in self.item.external_references:
            if ref.url:
                sponsor_message.add_row(
                    f"{ref.source_name} - {ref.url} - {ref.description}",
                    f"[u blue link={ref.url}]",
                )

        if self.item.description:
            intro_message = Text.from_markup(self.item.description.replace("[", ""))
        else:
            intro_message = Text.from_markup("UNKNOWN")

        message = Table.grid(padding=1)
        message.add_column()
        message.add_column(no_wrap=True)
        message.add_row(intro_message, sponsor_message)

        message_panel = Panel(
            Align.center(
                Group(intro_message, "\n", Align.center(sponsor_message)),
                vertical="middle",
            ),
            box=box.ROUNDED,
            padding=(1, 2),
            title="[b red]General Information",
            border_style="bright_blue",
        )
        return message_panel
